/*
 * glChess - A 3D chess interface
 *
 * Copyright (C) 2002  Robert  Ancell <bob27@users.sourceforge.net>
 *                     Michael Duelli <duelli@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "global.h"
#include "engine.h"
#include "dialog.h"
#include "config.h"

#include "logo.xpm"

extern int engine;
extern int promotion;

/* This shows a dialog with a messages */
void engine_dialog(char *msg)
{
  GtkWidget *dialog;
  GtkWidget *hbox, *label, *ok_button;

  dialog = gtk_dialog_new();

  gtk_window_set_title(GTK_WINDOW(dialog), "Info");
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

  hbox = gtk_hbox_new(FALSE, 10);

  /* Create the label */
  label = gtk_label_new(msg);

  /* Create the button */
  ok_button = gtk_button_new_with_label("OK");
  gtk_signal_connect_object(GTK_OBJECT(ok_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    GTK_OBJECT(dialog));

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, TRUE, TRUE,
		     10);
  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     ok_button, TRUE, TRUE, 0);

  gtk_widget_show_all(dialog);
}

/* The help about screen */
void about_dialog(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  GtkWidget *dialog = NULL;
  GtkWidget *hbox, *label, *button;

  GdkPixmap *gdkpix;
  GdkBitmap *mask;
  GtkWidget *pixmap;

  /* Quit if already open */
  if (dialog != NULL)
    return;

  dialog = gtk_dialog_new();
  hbox = gtk_hbox_new(FALSE, 10);

  gtk_window_set_title(GTK_WINDOW(dialog), "About glChess");
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

  gtk_signal_connect(GTK_OBJECT(dialog),
		     "unrealize", GTK_SIGNAL_FUNC(exit_dialog), &dialog);
  label =
      gtk_label_new("Version " VERSION "\n"
		    "\n"
		    "Build date " __DATE__ "\n"
		    "Build time " __TIME__ "\n"
		    "\n"
		    "Robert  Ancell <bob27@users.sourceforge.net>\n"
		    "Michael Duelli <duelli@users.sourceforge.net>\n\n"
		    "http://glchess.sf.net");


  /* Create the image */
  gtk_widget_realize(dialog);
  gdkpix = gdk_pixmap_create_from_xpm_d(dialog->window, &mask,
					NULL, (gchar **) logo_xpm);
  pixmap = gtk_pixmap_new(gdkpix, mask);

  /* Pack the image */
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), pixmap, TRUE,
		     TRUE, 10);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, TRUE, TRUE,
		     10);
  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 10);

  button = gtk_button_new_with_label("Close");
  gtk_signal_connect_object(GTK_OBJECT(button),
			    "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    GTK_OBJECT(dialog));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     button, TRUE, TRUE, 10);

  gtk_widget_show_all(dialog);
}

void quit_dialog(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  GtkWidget *dialog = NULL;
  GtkWidget *hbox, *label;
  GtkWidget *yes_btn, *no_btn;

  /* Quit if already open */
  if (dialog != NULL)
    return;

  dialog = gtk_dialog_new();
  hbox = gtk_hbox_new(FALSE, 10);

  gtk_window_set_title(GTK_WINDOW(dialog), "Quit glChess ?");
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

  gtk_signal_connect(GTK_OBJECT(dialog),
		     "unrealize", GTK_SIGNAL_FUNC(exit_dialog), &dialog);
  label = gtk_label_new("Do you really want to quit glChess ?");

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, TRUE, TRUE,
		     10);
  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 10);

  yes_btn = gtk_button_new_with_label("Yes");
  gtk_signal_connect(GTK_OBJECT(yes_btn), "clicked",
		     GTK_SIGNAL_FUNC(quit_program), glcwidget);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     yes_btn, TRUE, TRUE, 10);


  no_btn = gtk_button_new_with_label("No");
  gtk_signal_connect_object(GTK_OBJECT(no_btn), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    GTK_OBJECT(dialog));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
		     no_btn, TRUE, TRUE, 10);

  gtk_widget_show_all(dialog);
}

/* Quits the program */
void quit_program(GtkWidget * widget, glChessWidget * glcwidget)
{
  close_cecp_interface(glcwidget->game);

  gtk_main_quit();
}

/* Set a dialog to NULL when quit */
void exit_dialog(GtkWidget * widget, GtkWidget ** dialog)
{
  *dialog = NULL;
}

void promotion_dialog(void)
{
  GtkWidget *dialog;

  GtkWidget *label, *button;
  GSList *radio_list = NULL;
  GtkWidget *radio1, *radio2, *radio3, *radio4;

  dialog = gtk_dialog_new();

  gtk_window_set_title(GTK_WINDOW(dialog), "Choose promotion piece");
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

  gtk_signal_connect(GTK_OBJECT(dialog), "unrealize",
		     GTK_SIGNAL_FUNC(exit_dialog), &dialog);

  promotion = QUEEN;
  radio1 = gtk_radio_button_new_with_label(radio_list, "Queen");
  radio_list = gtk_radio_button_group(GTK_RADIO_BUTTON(radio1));
  gtk_signal_connect(GTK_OBJECT(radio1), "toggled",
		     GTK_SIGNAL_FUNC(promotion_dialog_cb), (int *) QUEEN);

  radio2 = gtk_radio_button_new_with_label(radio_list, "Rook");
  radio_list = gtk_radio_button_group(GTK_RADIO_BUTTON(radio2));
  gtk_signal_connect(GTK_OBJECT(radio2), "toggled",
		     GTK_SIGNAL_FUNC(promotion_dialog_cb), (int *) ROOK);

  radio3 = gtk_radio_button_new_with_label(radio_list, "Knight");
  radio_list = gtk_radio_button_group(GTK_RADIO_BUTTON(radio3));
  gtk_signal_connect(GTK_OBJECT(radio3), "toggled",
		     GTK_SIGNAL_FUNC(promotion_dialog_cb), (int *) KNIGHT);

  radio4 = gtk_radio_button_new_with_label(radio_list, "Bishop");
  radio_list = gtk_radio_button_group(GTK_RADIO_BUTTON(radio4));
  gtk_signal_connect(GTK_OBJECT(radio4), "toggled",
		     GTK_SIGNAL_FUNC(promotion_dialog_cb), (int *) BISHOP);

  label = gtk_label_new("Your pawn reached the opponent's baseline.\n"
			"Choose your promotion piece:");

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE,
		     0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), radio1, TRUE, TRUE,
		     0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), radio2, TRUE, TRUE,
		     0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), radio3, TRUE, TRUE,
		     0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), radio4, TRUE, TRUE,
		     0);

  button = gtk_button_new_with_label("OK");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    GTK_OBJECT(dialog));
  gtk_box_pack_end(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE,
		   TRUE, 0);

  gtk_widget_show_all(dialog);
}

/* Sets promotion piece */
void promotion_dialog_cb(GtkWidget * radio_btn, int *piece)
{
  if (GTK_TOGGLE_BUTTON(radio_btn)->active)
    promotion = (int) piece;
}

void binary_dialog(Game * game)
{
  GtkWidget *dialog;
  GtkWidget *ok_btn, *save_btn;

  FileSelection *entry1 = malloc(sizeof(FileSelection));
  FileSelection *entry2 = malloc(sizeof(FileSelection));

  GtkWidget *label1, *label2;
  GtkWidget *check1, *check2;

  dialog = gtk_dialog_new();
  gtk_signal_connect(GTK_OBJECT(dialog), "unrealize",
		     GTK_SIGNAL_FUNC(exit_dialog), &dialog);
  gtk_container_set_border_width(GTK_CONTAINER(dialog), 2);
  gtk_window_set_title(GTK_WINDOW(dialog), "Enter paths");
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

  /* Crafty stuff */
  label1 = gtk_label_new("Path for the crafty binary:");
  entry1->entry = gtk_entry_new();
  check1 = gtk_check_button_new_with_label("Don't have Crafty engine");

  gtk_signal_connect(GTK_OBJECT(entry1->entry), "unrealize",
		     GTK_SIGNAL_FUNC(save_crafty_path), game);
  gtk_signal_connect(GTK_OBJECT(check1), "clicked",
		     GTK_SIGNAL_FUNC(toggle_use_crafty), entry1);

  if (game->path_crafty)
    gtk_entry_set_text(GTK_ENTRY(entry1->entry), game->path_crafty);
  else
  {
    gtk_entry_set_text(GTK_ENTRY(entry1->entry), "do not use");
    game->path_crafty = g_strdup("do not use");
  }

  if (!strcmp(game->path_crafty, "do not use"))
  {
    GTK_TOGGLE_BUTTON(check1)->active = TRUE;
    gtk_entry_set_editable(GTK_ENTRY(entry1->entry), FALSE);
  }

  /* GNUChessX stuff */
  label2 = gtk_label_new("Path for the gnuchessx binary:");
  entry2->entry = gtk_entry_new();
  check2 = gtk_check_button_new_with_label("Don't have GNUChessX engine");

  gtk_signal_connect(GTK_OBJECT(entry2->entry), "unrealize",
		     GTK_SIGNAL_FUNC(save_gnuchessx_path), game);
  gtk_signal_connect(GTK_OBJECT(check2), "clicked",
		     GTK_SIGNAL_FUNC(toggle_use_gnuchessx), entry2);

  if (game->path_gnuchessx)
    gtk_entry_set_text(GTK_ENTRY(entry2->entry), game->path_gnuchessx);
  else
  {
    gtk_entry_set_text(GTK_ENTRY(entry2->entry), "do not use");
    game->path_gnuchessx = g_strdup("do not use");
  }

  if (!strcmp(game->path_gnuchessx, "do not use"))
  {
    GTK_TOGGLE_BUTTON(check2)->active = TRUE;
    gtk_entry_set_editable(GTK_ENTRY(entry2->entry), FALSE);
  }

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label1, TRUE, TRUE,
		     0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), entry1->entry,
		     TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), check1, TRUE, TRUE,
		     0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label2, TRUE, TRUE,
		     0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), entry2->entry,
		     TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), check2, TRUE, TRUE,
		     0);
  /* Buttons */
  ok_btn = gtk_button_new_with_label("OK");
  save_btn = gtk_button_new_with_label("Save");

  gtk_signal_connect_object(GTK_OBJECT(ok_btn), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    GTK_OBJECT(dialog));
  gtk_signal_connect(GTK_OBJECT(save_btn), "clicked",
		     GTK_SIGNAL_FUNC(save_paths), game);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), ok_btn,
		     TRUE, TRUE, 0);
  gtk_box_pack_end(GTK_BOX(GTK_DIALOG(dialog)->action_area), save_btn,
		   TRUE, TRUE, 0);

  gtk_widget_show_all(dialog);
}

void save_paths(GtkWidget * save_btn, Game * game)
{
  char fname[50];

  snprintf(fname, 50, "%s/.glchessrc", getenv("HOME"));
  save_rc(game, fname);
}

void save_crafty_path(GtkWidget * entry1, Game * game)
{
  gchar *path;

  path = gtk_entry_get_text(GTK_ENTRY(entry1));

  if (game->path_crafty)
    g_free(game->path_crafty);

  if (!path)
    game->path_crafty = g_strdup("do not use");
  else
    game->path_crafty = g_strdup(path);

  if (game->cecp.debug)
    debug_output("DEBUG ( %11s line %3d ): Crafty path is %s\n", __FILE__,
		 __LINE__, path);
}

void save_gnuchessx_path(GtkWidget * entry2, Game * game)
{
  gchar *path;

  path = gtk_entry_get_text(GTK_ENTRY(entry2));

  if (game->path_gnuchessx)
    g_free(game->path_gnuchessx);

  if (!path)
    game->path_gnuchessx = g_strdup("do not use");
  else
    game->path_gnuchessx = g_strdup(path);

  if (game->cecp.debug)
    debug_output("DEBUG ( %11s line %3d ): GNUChessX path is %s\n",
		 __FILE__, __LINE__, path);
}

void toggle_use_crafty(GtkWidget * check1, FileSelection * entry1)
{
  if (GTK_TOGGLE_BUTTON(check1)->active)
  {
    gtk_entry_set_text(GTK_ENTRY(entry1->entry), "do not use");
    gtk_entry_set_editable(GTK_ENTRY(entry1->entry), FALSE);
  } else
  {
    char *bin;

    gtk_entry_set_editable(GTK_ENTRY(entry1->entry), TRUE);
    if ((bin = binary_path("crafty")))
      gtk_entry_set_text(GTK_ENTRY(entry1->entry), bin);
    else
    {
      gtk_entry_set_text(GTK_ENTRY(entry1->entry),
			 "Please insert binary path");

      debug_output
	  ("DEBUG ( %11s line %3d ): Calling file selection dialog for Crafty\n",
	   __FILE__, __LINE__);

      /* Call file dialog */
      entry1->filesel =
	  gtk_file_selection_new("Please choose the binary path");
      gtk_window_set_modal(GTK_WINDOW(entry1->filesel), TRUE);

      gtk_signal_connect(GTK_OBJECT(entry1->filesel),
			 "unrealize",
			 GTK_SIGNAL_FUNC(exit_dialog), &entry1->filesel);

      gtk_signal_connect(GTK_OBJECT
			 (GTK_FILE_SELECTION(entry1->filesel)->ok_button),
			 "clicked", GTK_SIGNAL_FUNC(set_entry), entry1);

      gtk_signal_connect_object(GTK_OBJECT
				(GTK_FILE_SELECTION(entry1->filesel)->
				 ok_button), "clicked",
				GTK_SIGNAL_FUNC(gtk_widget_destroy),
				GTK_OBJECT(entry1->filesel));

      gtk_signal_connect_object(GTK_OBJECT
				(GTK_FILE_SELECTION(entry1->filesel)->
				 cancel_button), "clicked",
				GTK_SIGNAL_FUNC(gtk_widget_destroy),
				GTK_OBJECT(entry1->filesel));

      gtk_widget_show_all(entry1->filesel);
    }

    free(bin);
  }
}

void set_entry(GtkWidget * ok_btn, FileSelection * entry)
{
  if (gtk_file_selection_get_filename(GTK_FILE_SELECTION(entry->filesel))
      != NULL)
    gtk_entry_set_text(GTK_ENTRY(entry->entry),
		       gtk_file_selection_get_filename(GTK_FILE_SELECTION
						       (entry->filesel)));
}

void toggle_use_gnuchessx(GtkWidget * check2, FileSelection * entry2)
{
  if (GTK_TOGGLE_BUTTON(check2)->active)
  {
    gtk_entry_set_text(GTK_ENTRY(entry2->entry), "do not use");
    gtk_entry_set_editable(GTK_ENTRY(entry2->entry), FALSE);
  } else
  {
    char *bin;

    gtk_entry_set_editable(GTK_ENTRY(entry2->entry), TRUE);
    if ((bin = binary_path("gnuchess")))
      gtk_entry_set_text(GTK_ENTRY(entry2->entry), bin);
    else
    {
      gtk_entry_set_text(GTK_ENTRY(entry2->entry),
			 "Please insert binary path");

      debug_output
	  ("DEBUG ( %11s line %3d ): Calling file selection dialog for GNU ChessX\n",
	   __FILE__, __LINE__);

      /* Call file dialog */
      entry2->filesel =
	  gtk_file_selection_new("Please choose the binary path");
      gtk_window_set_modal(GTK_WINDOW(entry2->filesel), TRUE);

      gtk_signal_connect(GTK_OBJECT(entry2->filesel),
			 "unrealize",
			 GTK_SIGNAL_FUNC(exit_dialog), &entry2->filesel);

      gtk_signal_connect(GTK_OBJECT
			 (GTK_FILE_SELECTION(entry2->filesel)->ok_button),
			 "clicked", GTK_SIGNAL_FUNC(set_entry), entry2);

      gtk_signal_connect_object(GTK_OBJECT
				(GTK_FILE_SELECTION(entry2->filesel)->
				 ok_button), "clicked",
				GTK_SIGNAL_FUNC(gtk_widget_destroy),
				GTK_OBJECT(entry2->filesel));

      gtk_signal_connect_object(GTK_OBJECT
				(GTK_FILE_SELECTION(entry2->filesel)->
				 cancel_button), "clicked",
				GTK_SIGNAL_FUNC(gtk_widget_destroy),
				GTK_OBJECT(entry2->filesel));

      gtk_widget_show_all(entry2->filesel);
    }

    free(bin);
  }
}
