/*
 * glChess - A 3D chess interface
 *
 * Copyright (C) 2002  Robert  Ancell <bob27@users.sourceforge.net>
 *                     Michael Duelli <duelli@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <string.h>

#include "global.h"
#include "menu.h"
#include "draw.h"
#include "splash.h"
#include "player.h"
#include "interface.h"
#include "config.h"
#include "move.h"
#include "engine.h"
#include "models.h"
#include "dialog.h"
#include "prefs.h"

extern int engine;
extern int promotion;
extern int *move_list;
extern float piece_size;

GtkWidget *generate_menus(glChessWidget * glcwidget,
			  GtkWidget ** view_radio, GtkWidget ** fps_radio)
{
  Game *game = glcwidget->game;


  /* Make things line up better */
#define GPA "/Game/Play a game against..."
#define VM  "/View/Mode"
#define VR  "/View/Reflections"
#define VPS "/View/Piece size"
#define TPS toggle_piece_size
#define EG  "/Engine"

  /* The menus: */
  /* *INDENT-OFF* */
  static GtkItemFactoryEntry menu_items[] = {
    {"/_Game",                    NULL, NULL,            0,   "<Branch>"},
    {"/Game/tear",                NULL, NULL,            0,   "<Tearoff>"},
    {"/Game/_Play a game against...", NULL, NULL,        0,   "<Branch>"},
    {GPA "/tear",                 NULL, NULL,            0,   "<Tearoff>"},
    {GPA "/_Crafty",              "C",  start_game,      1,   "<Item>"},
    {GPA "/_GNU Chess 4",         "G",  start_game,      2,   "<Item>"},
    {GPA "/_Yourself (no engine)","Y",  start_game,      3,   "<Item>"},
    {"/Game/_Resign",             "R",  resign_game,     1,   "<Item>"},
    {"/Game/sep1",                NULL, NULL,            0,   "<Separator>"},
    {"/Game/_Quit",               "Q",  quit_dialog,     1,   "<Item>"},
    {"/_Settings",                NULL, NULL,            0,   "<Branch>"},
    {"/Settings/tear",            NULL, NULL,            0,   "<Tearoff>"},
    {"/Settings/_Preferences...", "<ctl>F", preferences, 1,   "<Item>"},
    {"/Settings/Edit binary paths", NULL, binary_dialog_cb, 0, "<Item>"}, 
    {"/Settings/_Save Settings",  NULL, save_settings,   1,   "<Item>"},
    {"/_View",                    NULL, NULL,            0,   "<Branch>"},
    {"/View/tear",                NULL, NULL,            0,   "<Tearoff>"},
    {"/View/_Paused",             "P",  toggle_pause,    1,   "<CheckItem>"},
    {"/View/_Mode",               NULL, NULL,            0,   "<Branch>"},
    {"/View/Mode/tear",           NULL, NULL,            0,   "<Tearoff>"},
    {VM "/_Free",                 "1",  set_view_mode,   MODE_FREE,      "<RadioItem>"},
    {VM "/_Rotating",             "2",  set_view_mode,   MODE_ROTATING,  VM "/Free"},
    {VM "/_Ortho",                "3",  set_view_mode,   MODE_ORTHO,     VM "/Free"},
    {VM "/_Trackball",            "4",  set_view_mode,   MODE_TRACKBALL, VM "/Free"},
    {"/View/Piece size",          NULL, NULL,            0,   "<Branch>"},
    {"/View/Piece size/tear",     NULL, NULL,            0,   "<Tearoff>"},
    {VPS "/100% (normal)",        NULL, TPS,             0,   "<RadioItem>"},
    {VPS "/ 75% (medium)",        NULL, TPS,             1,   VPS "/100% (normal)"},
    {VPS "/ 50% (small )",        NULL, TPS,             2,   VPS "/100% (normal)"},
    {VPS "/ 25% (little)",        NULL, TPS,             3,   VPS "/100% (normal)"},
    {"/View/Max _FPS",            NULL, NULL,            0,   "<Branch>"},
    {"/View/Max FPS/tear",        NULL, NULL,            0,   "<Tearoff>"},
    {"/View/Max FPS/_1 fps",      NULL, set_max_fps,     1,   "<RadioItem>"},
    {"/View/Max FPS/_2 fps",      NULL, set_max_fps,     2,   "/View/Max FPS/1 fps"},
    {"/View/Max FPS/_4 fps",      NULL, set_max_fps,     4,   "/View/Max FPS/1 fps"},
    {"/View/Max FPS/_6 fps",      NULL, set_max_fps,     6,   "/View/Max FPS/1 fps"},
    {"/View/Max FPS/_8 fps",      NULL, set_max_fps,     8,   "/View/Max FPS/1 fps"},
    {"/View/Max FPS/_10 fps",     NULL, set_max_fps,     10,  "/View/Max FPS/1 fps"},
    {"/View/Max FPS/_15 fps",     NULL, set_max_fps,     15,  "/View/Max FPS/1 fps"},
    {"/View/Max FPS/_20 fps",     NULL, set_max_fps,     20,  "/View/Max FPS/1 fps"},
    {"/View/Max FPS/_30 fps",     NULL, set_max_fps,     30,  "/View/Max FPS/1 fps"},
    {"/View/Max FPS/_Unlimited",  NULL, set_max_fps,     666, "/View/Max FPS/1 fps"},
    {"/View/sep1",                NULL, NULL,            0,   "<Separator>"},
    {"/View/_Lighting",           "L",  toggle_lighting, 1,   "<CheckItem>"},
    {"/View/_Shading",            "H",  toggle_shading,  1,   "<CheckItem>"},
    {"/View/_Reflections",        NULL, NULL,            0,   "<Branch>"},
    {"/View/Reflections/tear",    NULL, NULL,            0,   "<Tearoff>"},
    {VR "/_Off",                  NULL, set_reflect,     0,   "<RadioItem>"},
    {VR "/_Quick hack",           NULL, set_reflect,     1,   VR "/Off"},
    {VR "/_Stencil buffer",       NULL, set_reflect,     2,   VR "/Off"},
    {"/View/_Textures",           "K",  toggle_texture,  1,   "<CheckItem>"},
    {"/View/_Fog",                "M",  toggle_fog,      1,   "<CheckItem>"},
    {"/View/Show _Coords",        "N",  toggle_coords,   1,   "<CheckItem>"},
    {"/_Engine",                  NULL, NULL,            0,   "<Branch>"},
    {EG "/tear",                  NULL, NULL,            0,   "<Tearoff>"},
    {EG "/Beep on illegal move",  NULL, toggle_beep,     0,   "<CheckItem>"},
    {EG "/Debug engine on console", NULL, toggle_debug,  0,   "<CheckItem>"},
    {EG "/Engine plays white", "W", toggle_color,        0,   "<Item>"},
    {EG "/Engine plays black", "B", toggle_color,        1,   "<Item>"},
    {"/_Help",                    NULL, NULL,            0,   "<LastBranch>"},
    {"/Help/tear",                NULL, NULL,            0,   "<Tearoff>"},
    {"/Help/_About glChess...",   NULL, about_dialog,    0,   "<Item>"},
  };
  /* *INDENT-ON* */

  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);

  GtkWidget *check_item = NULL;

  /* Make the item factory */
  accel_group = gtk_accel_group_new();
  glcwidget->item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR,
						 "<main>", accel_group);

  /* Convert the itemfactory to menu items */
  gtk_item_factory_create_items(glcwidget->item_factory,
				nmenu_items, menu_items, glcwidget);

  /* Set the defaults on the view options */
  check_item =
      gtk_item_factory_get_widget(glcwidget->item_factory,
				  "/View/Lighting");
  if (game->is_light)
    GTK_CHECK_MENU_ITEM(check_item)->active = TRUE;

  check_item =
      gtk_item_factory_get_widget(glcwidget->item_factory,
				  "/View/Shading");
  if (game->is_smooth)
    GTK_CHECK_MENU_ITEM(check_item)->active = TRUE;

  check_item =
      gtk_item_factory_get_widget(glcwidget->item_factory,
				  "/View/Textures");
  if (game->is_texture)
    GTK_CHECK_MENU_ITEM(check_item)->active = TRUE;

  check_item =
      gtk_item_factory_get_widget(glcwidget->item_factory, "/View/Fog");
  if (game->is_fog)
    GTK_CHECK_MENU_ITEM(check_item)->active = TRUE;

  check_item =
      gtk_item_factory_get_widget(glcwidget->item_factory,
				  "/View/Show Coords");
  if (game->is_coord)
    GTK_CHECK_MENU_ITEM(check_item)->active = TRUE;

  switch (game->is_reflect)
  {
  case 0:
    check_item =
	gtk_item_factory_get_widget(glcwidget->item_factory, VR "/Off");
    break;
  case 1:
    check_item =
	gtk_item_factory_get_widget(glcwidget->item_factory,
				    VR "/Quick hack");
    break;
  case 2:
    check_item =
	gtk_item_factory_get_widget(glcwidget->item_factory,
				    VR "/Stencil buffer");
    break;
  default:
    break;
  }
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(check_item), TRUE);


  switch (game->camera->mode)
  {
  case MODE_FREE:
    check_item =
	gtk_item_factory_get_widget(glcwidget->item_factory, VM "/Free");
    break;
  case MODE_ROTATING:
    check_item =
	gtk_item_factory_get_widget(glcwidget->item_factory,
				    VM "/Rotating");
    break;
  case MODE_ORTHO:
    check_item =
	gtk_item_factory_get_widget(glcwidget->item_factory, VM "/Ortho");
    break;
  case MODE_TRACKBALL:
  default:
    check_item =
	gtk_item_factory_get_widget(glcwidget->item_factory,
				    VM "/Trackball");
    break;
  }
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(check_item), TRUE);

  check_item = gtk_item_factory_get_widget(glcwidget->item_factory,
					   "/Engine/Beep on illegal move");
  if (game->cecp.beep_signal)
    GTK_CHECK_MENU_ITEM(check_item)->active = TRUE;

  check_item = gtk_item_factory_get_widget(glcwidget->item_factory,
					   EG "/Debug engine on console");
  if (game->cecp.debug)
    GTK_CHECK_MENU_ITEM(check_item)->active = TRUE;

  check_item = gtk_item_factory_get_widget(glcwidget->item_factory,
					   "/View/Max FPS/Unlimited");
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(check_item), TRUE);

  check_item = gtk_item_factory_get_widget(glcwidget->item_factory,
					   "/View/Piece size/100% (normal)");
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(check_item), TRUE);


  /* Add the shortcuts */
  gtk_window_add_accel_group(GTK_WINDOW(glcwidget->window), accel_group);

  /* Return the root menu */
  return gtk_item_factory_get_widget(glcwidget->item_factory, "<main>");
}

/* Starts the game */
void start_game(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  Game *game = glcwidget->game;


  if (engine == -1)
  {
    engine = num;

    switch (engine)
    {
    case CRAFTY:
      /* Check whether user said to don't have Crafty */
      if (!strcmp(game->path_crafty, "do not use"))
      {
	engine_dialog("You claimed to don't have Crafty installed.\n"
		      "If you have it now go to \"Settings/Edit paths\".");
	engine = -1;
	return;
      }

      /* Check the user defined path for Crafty */
      if (fopen(game->path_crafty, "r") == NULL)
      {
	engine_dialog("The path defined for the Crafty binary is wrong.\n"
		      "Please go to \"Settings/Edit paths\".");
	engine = -1;
	return;
      }

      break;
    case GNUCHESSX:
      /* Check whether user said to don't have GNUChessX */
      if (!strcmp(game->path_gnuchessx, "do not use"))
      {
	engine_dialog("You claimed to don't have GNUChessX installed.\n"
		      "If you have it now go to \"Settings/Edit paths\".");
	engine = -1;
	return;
      }

      /* Check the user defined path for GNUChessX */
      if (fopen(game->path_gnuchessx, "r") == NULL)
      {
	engine_dialog
	    ("The path defined for the GNUChessX binary is wrong.\n"
	     "Please go to \"Settings/Edit paths\".");
	engine = -1;
	return;
      }

      break;
    case HUMAN:
    default:
      break;
    }
    end_splash(widget, glcwidget);


    /* Generate internal move list */
    move_list_create();
    /* Start a game */
    init_game(glcwidget);
  } else
  {
    engine_dialog("Please quit the current game\n"
		  "before starting another one");
  }
}

/* Finish a game */
void resign_game(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  /* Free move list */
  move_list_free();

  /* Reset engine */
  engine = -1;

  end_game(glcwidget);
}

void toggle_pause(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  Game *game = glcwidget->game;

  game->is_paused = !game->is_paused;
}

void binary_dialog_cb(glChessWidget * glcwidget, guint num,
		      GtkWidget * widget)
{
  Game *game = glcwidget->game;

  binary_dialog(game);
}


/* Saves the current settings to ~/.glchessrc */
void
save_settings(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  Game *game = glcwidget->game;
  char fname[50];

  snprintf(fname, 50, "%s/.glchessrc", getenv("HOME"));
  save_rc(game, fname);
}

/* Toggles use of lighting */
void
toggle_lighting(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  Game *game = glcwidget->game;

  game->is_light = !game->is_light;
  if (game->is_light)
    glEnable(GL_LIGHTING);
  else
    glDisable(GL_LIGHTING);
  post_redisplay(game);
}

/* Toggles shaded polygons */
void
toggle_shading(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  Game *game = glcwidget->game;

  game->is_smooth = !game->is_smooth;
  if (game->is_smooth)
    glShadeModel(GL_SMOOTH);
  else
    glShadeModel(GL_FLAT);
  post_redisplay(game);
}

/* Toggles use of textures */
void
toggle_texture(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  Game *game = glcwidget->game;

  game->is_texture = !game->is_texture;
  if (game->is_texture)
    glEnable(GL_TEXTURE_2D);
  else
    glDisable(GL_TEXTURE_2D);
  post_redisplay(game);
}

/* Toggles the use of fog */
void toggle_fog(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  Game *game = glcwidget->game;

  game->is_fog = !game->is_fog;
  if (game->is_fog)
    glEnable(GL_FOG);
  else
    glDisable(GL_FOG);
  post_redisplay(game);
}

/* Toggles the use of co-ordinates */
void
toggle_coords(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  Game *game = glcwidget->game;

  game->is_coord = !game->is_coord;
  post_redisplay(game);
}

/* Toggles beep on illegal move */
void toggle_beep(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  Game *game = glcwidget->game;

  game->cecp.beep_signal = !game->cecp.beep_signal;
}

/* Toggles debuging for engines */
void toggle_debug(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  Game *game = glcwidget->game;

  game->cecp.debug = !game->cecp.debug;
}

/* Toggles the engine color */
void toggle_color(glChessWidget * glcwidget, guint num, GtkWidget * widget)
{
  Game *game = glcwidget->game;
  int engine_start[2];
  int engine_target[2];

  if (engine == CRAFTY || engine == GNUCHESSX)
  {
    switch (num)
    {
    case 0:
      if (game->cecp.ai_player == BLACK PLAYER)
      {
	game->current_player = game->white_player;
	game->cecp.ai_player = WHITE PLAYER;
	write_to_engine("go", game);
	parse_move_from_engine(game, read_from_engine(game),
			       engine_start, engine_target);
	game->selected[0] = engine_start[0];
	game->selected[1] = engine_start[1];
	move_piece(game, engine_target);
	game->selected[0] = -1;
      } else if (game->cecp.debug)
	debug_output
	    ("DEBUG ( %11s line %3d ): Engine already plays white!\n",
	     __FILE__, __LINE__);
      break;
    case 1:
      if (game->cecp.ai_player == WHITE PLAYER)
      {
	game->current_player = game->black_player;
	game->cecp.ai_player = BLACK PLAYER;
	write_to_engine("go", game);
	parse_move_from_engine(game, read_from_engine(game),
			       engine_start, engine_target);
	game->selected[0] = engine_start[0];
	game->selected[1] = engine_start[1];
	move_piece(game, engine_target);
	game->selected[0] = -1;
      } else if (game->cecp.debug)
	debug_output
	    ("DEBUG ( %11s line %3d ): Engine already plays black!\n",
	     __FILE__, __LINE__);
      break;
    }
  } else
    engine_dialog("You must start a game against an engine,\n"
		  "before you make this change");
}

/* Toggles the piece size */
void toggle_piece_size(glChessWidget * glcwidget, guint num,
		       GtkWidget * widget)
{
  piece_size = (4.0 - (float) num) / 4.0;

  gen_model_lists();
}

/* Sets the view mode */
void
set_view_mode(glChessWidget * glcwidget, guint mode, GtkWidget * widget)
{
  Game *game = glcwidget->game;

  game->camera->mode = mode;
  reset_perspective(game);

  post_redisplay(game);
}

void set_reflect(glChessWidget * glcwidget, guint value,
		 GtkWidget * widget)
{
  glcwidget->game->is_reflect = value;
}

/* Sets what the minimum dt is, so not all availiable cpu is used */
void set_max_fps(glChessWidget * glcwidget, guint fps, GtkWidget * widget)
{
  if (fps == 666)
    glcwidget->game->min_dt = 0;
  else
    glcwidget->game->min_dt = 1.0 / (float) fps;
}
